package gssim.gssimstudio.flex.app.editor;

import java.io.*;
import java.util.*;

import org.vinetoolkit.ResourceManager;
import org.vinetoolkit.ServiceContext;
import org.vinetoolkit.Session;

import org.vinetoolkit.web.spi.annotation.BlazeDsDestination;
import org.vinetoolkit.web.spi.annotation.BlazeDsMethod;
import org.vinetoolkit.web.flex.blazeds.shared.utils.VineFlexUtilClass;
import org.vinetoolkit.web.portlet.spi.annotation.PortletDisplayName;
import org.vinetoolkit.role.VineRoleManager;
import org.vinetoolkit.spi.util.ServiceLog;

import flex.messaging.FlexContext;
import org.vinetoolkit.web.spi.annotation.BlazeDsDestination;
import org.vinetoolkit.web.spi.annotation.BlazeDsMethod;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.disk.DiskFileItem;
import org.vinetoolkit.io.*;
import org.vinetoolkit.io.sets.FileLocationSet;
import org.vinetoolkit.ServiceContext;
import org.vinetoolkit.Location;
import org.vinetoolkit.task.TaskMonitor;
import org.vinetoolkit.task.TaskOptions;
import org.vinetoolkit.task.TaskException;
import org.vinetoolkit.localhost.io.FileSystemResource;
import org.vinetoolkit.localhost.io.FileSystemManager;
import org.vinetoolkit.spi.util.ServiceLog;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Timestamp;

import java.security.Principal;

import gssim.db.dao.data.ApplicationProperty;
import gssim.db.find.Finder;
import gssim.db.mysql.ApplicationPropertyDAO;
import gssim.db.mysql.MySQLDAOFactory;

import java.net.MalformedURLException;

import javax.servlet.ServletContext;

/**
 * @author Mateusz Cichenski, Jarek Szymczak
 */
@BlazeDsDestination(locales = {"en_US","pl_PL"}, names = {"GSSim Studio Configuration","GSSim Studio - ustawienia"}, descriptions = {"Application for configuring GSSim Studio","Aplikacja konfiguracji GSSim Studio"})
@PortletDisplayName(locales = {"en", "pl"}, values = {"GSSimStudio Configuration Application", "Aplikacja GSSimStudio - ustawienia"})
public class GSSimStudioConfig extends DatabaseHelper {

	private Boolean developement = true;

	private static MySQLDAOFactory factory;

	/**
	 * Logger instance for this class
	 */
    private ServiceLog log = ServiceLog.getInstance(GSSimStudioConfig.class);
	
	@BlazeDsMethod()
	public GSSimStudioConfigModel getApplicationPropertyTypesFromDb() {
		GSSimStudioConfigModel model = (GSSimStudioConfigModel) VineFlexUtilClass.getUIModel();
		Connection connection = null;
		try {
			connection = factory().createDBConnection(dbAddress, dbUser,
					dbPassword);
			ApplicationPropertyDAO applicationPropertyDAO = factory.createApplicationProperty();
			
			List<String> list = applicationPropertyDAO.getTypes();
			
			model.setPropertyList(list);
			model.setError("");

		} catch (SQLException e) {
			setSqlError("Text properties", model, e);
		} catch (InstantiationException e) {
			setDbConnectionError(model, e);		
		} catch (IllegalAccessException e) {
			setDbConnectionError(model, e);	
		} catch (ClassNotFoundException e) {
			setDbConnectionError(model, e);	
		} catch (Exception e) {
			setError("Unexpected error occured", model, e);	
		} finally {
			closeConnection(connection, model);
		}

		return getPropertyValue(model.getPropertyList().get(0));
	}
	
	@BlazeDsMethod()
	public GSSimStudioConfigModel getPropertyValue(String property) {
		GSSimStudioConfigModel model = (GSSimStudioConfigModel) VineFlexUtilClass.getUIModel();
		Connection connection = null;
		try {
			connection = factory().createDBConnection(dbAddress, dbUser,
					dbPassword);
			ApplicationPropertyDAO applicationPropertyDAO = factory.createApplicationProperty();
			Finder finder = Finder.getApplicationPropertyFinder(property);
			List<ApplicationProperty> list = applicationPropertyDAO.find(finder);
			if (list.isEmpty()) ;
			else {
				model.setValue(list.get(0).getValue());
				model.setError("");
			}
			
		} catch (SQLException e) {
			setSqlError("Text properties", model, e);
		} catch (InstantiationException e) {
			setDbConnectionError(model, e);		
		} catch (IllegalAccessException e) {
			setDbConnectionError(model, e);	
		} catch (ClassNotFoundException e) {
			setDbConnectionError(model, e);	
		} catch (Exception e) {
			setError("Unexpected error occured", model, e);	
		} finally {
			closeConnection(connection, model);
		}
		
		return model;
	}
	
	@BlazeDsMethod()
	public GSSimStudioConfigModel saveProperty(String property, String value) throws IOException {
		GSSimStudioConfigModel model = (GSSimStudioConfigModel) VineFlexUtilClass.getUIModel();
		Connection connection = null;
		try {
			connection = factory().createDBConnection(dbAddress, dbUser,
					dbPassword);
			ApplicationPropertyDAO applicationPropertyDAO = factory.createApplicationProperty();
			ApplicationProperty ap = new ApplicationProperty();
			ap.setKey(property);
			ap.setValue(value);
			applicationPropertyDAO.insert(ap, true);
			model.setError("Succesfully added: " + property + "=" + value);
			
		} catch (SQLException e) {
			setSqlError("Text properties", model, e);
		} catch (InstantiationException e) {
			setDbConnectionError(model, e);		
		} catch (IllegalAccessException e) {
			setDbConnectionError(model, e);	
		} catch (ClassNotFoundException e) {
			setDbConnectionError(model, e);	
		} catch (Exception e) {
			setError("Unexpected error occured", model, e);	
		} finally {
			closeConnection(connection, model);
		}
		
		return model;
	}
	
	private void closeConnection(Connection connection, GSSimStudioConfigModel model) {
		String error = model.getError();
		
		if (connection != null)
			try {
				connection.close();
			} catch (SQLException e) {
				error = model.getError();
				if (error == null || error.equals("")) {
					error = "Error while trying to close the SQL connection";
				} else {
					error += System.getProperty("line.separator");
					error += "Error while trying to close the SQL connection";
				}
			}

		model.setError(error);
	}

	private void setSqlError(String name, GSSimStudioConfigModel model, Exception e) {
		setError(name + ": an SQL exception occured while trying retrieve data from database", model, e);
	}
	
	private void setDbConnectionError(GSSimStudioConfigModel model, Exception e) {
		setError("An exception occured while trying to create MySQLDAOFactory", model, e);
	}
	
	private void setError(String messageForUser, GSSimStudioConfigModel model, Exception e) {
		
		String error = messageForUser;
		
		if(developement)
		{
			error += System.getProperty("line.separator");
			error += "Error details:";
			error += System.getProperty("line.separator");
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			PrintStream ps = new PrintStream(baos);
			e.printStackTrace(ps);
			String stackTrace = baos.toString();		
			error += stackTrace;
		}
		
		model.setError(error);
	}			
	
	private MySQLDAOFactory factory() throws InstantiationException, IllegalAccessException, ClassNotFoundException {
		if(factory == null)
			factory = new MySQLDAOFactory();
		return factory;
	}
	/* END OF DB Methods */
}